﻿Module UserInterface

    Public cfontNormal As ConsoleFont
    Public cfontWarning As ConsoleFont
    Public cfontError As ConsoleFont
    Public cfontMarked As ConsoleFont
    Public cfontTarget As ConsoleFont
    Public cfontTimestamp As ConsoleFont
    Public cfontEcho As ConsoleFont
    Public cfontChat As ConsoleFont

    Public ConsoleBackColor As Color
    Public InputBoxBackColor As Color
    Public InputBoxFont As ConsoleFont

    Delegate Sub delegate_AppendText(ByVal CData As Collection)
    Delegate Sub deleg_ShowBalloon(ByVal Text As String)

    Public ServerCrashedSound As String
    Public ServerStartedSound As String
    Public ServerStartFailedSound As String
    Public MangoRestartCrashedSound As String
    Public ChatSound As String

    Public EnableBalloons As Boolean
    Public BalloonsTimeOut As UInt16

    Public Structure ConsoleFont
        Dim ArgbColor As Integer
        Dim objFont As Font
    End Structure

    Public Structure ConsoleData
        Dim Text As String
        Dim CFont As ConsoleFont
    End Structure

    Public Sub InitUserInterface()
        cfontNormal = NewCFont("MS Reference Sans Serif", Color.White.ToArgb, 9)
        cfontWarning = NewCFont("MS Reference Sans Serif", Color.Yellow.ToArgb, 9)
        cfontError = NewCFont("MS Reference Sans Serif", Color.Red.ToArgb, 9)
        cfontMarked = NewCFont("MS Reference Sans Serif", Color.LightSkyBlue.ToArgb, 9)
        cfontTarget = NewCFont("MS Reference Sans Serif", Color.Orange.ToArgb, 9)
        cfontTimestamp = NewCFont("Arial", Color.LightGreen.ToArgb, 7)
        cfontEcho = NewCFont("MS Reference Sans Serif", Color.Cyan.ToArgb, 9)
        cfontChat = NewCFont("MS Reference Sans Serif", Color.HotPink.ToArgb, 9)
        InputBoxFont = NewCFont("MS Reference Sans Serif", Color.Blue.ToArgb, 9)
        InputBoxBackColor = Color.White
        ConsoleBackColor = Color.Black
    End Sub

    Public Function NewCFont(ByVal FontName As String, ByVal ArgbColor As Integer, ByVal Size As Byte) As ConsoleFont
        Dim CFont As ConsoleFont
        CFont.ArgbColor = ArgbColor
        CFont.objFont = New Font(FontName, Size, FontStyle.Regular, GraphicsUnit.Point, 204)
        Return CFont
    End Function

    Public Sub AppendText(ByVal CData As Collection)
        Dim deleg As New delegate_AppendText(AddressOf invoke_AppendText)
        MainWindow.Invoke(deleg, CData)
    End Sub

    Private Sub invoke_AppendText(ByVal CData As Collection)
        With MainWindow
            For Each CItem As ConsoleData In CData
                .txtConsole.SelectionLength = 0
                .txtConsole.SelectionStart = .txtConsole.Text.Length
                .txtConsole.SelectionColor = Color.FromArgb(CItem.CFont.ArgbColor)
                .txtConsole.SelectionFont = CItem.CFont.objFont
                .txtConsole.SelectedText = CItem.Text
            Next
            If Not .txtConsole.Focused Then
                .txtConsole.SelectionLength = 0
                .txtConsole.SelectionStart = MainWindow.txtConsole.Text.Length
                .txtConsole.ScrollToCaret()
            End If
        End With
    End Sub

    Public Function NewCData(ByVal Text As String, ByVal CFont As ConsoleFont) As ConsoleData
        Dim CData As New ConsoleData
        CData.Text = Text
        CData.CFont = CFont
        Return CData
    End Function

    Public Sub OutLog(ByVal ParamArray arrCData() As ConsoleData)
        Dim CData As New Collection()
        For Each objCData As ConsoleData In arrCData
            CData.Add(objCData)
        Next
        CData.Add(NewCData(vbCrLf, cfontNormal))
        AppendText(CData)
    End Sub

    Public Function TimeStamp() As ConsoleData
        Return NewCData(" (" & Now & ")", cfontTimestamp)
    End Function

    Public Function Target(ByVal Name As String) As ConsoleData
        Return NewCData(Name & ": ", cfontTarget)
    End Function

    Public Sub ShowBalloon(ByVal Text As String)
        If Not EnableBalloons Then Return
        If MainWindow.Visible And MainWindow.WindowState <> FormWindowState.Minimized Then Return
        Dim deleg As New deleg_ShowBalloon(AddressOf invoke_ShowBalloon)
        MainWindow.Invoke(deleg, Text)
    End Sub

    Private Sub invoke_ShowBalloon(ByVal Text As String)
        MainWindow.TrayIcon.BalloonTipIcon = ToolTipIcon.Warning
        MainWindow.TrayIcon.BalloonTipText = Text
        MainWindow.TrayIcon.BalloonTipTitle = "MangoRestart " & VersionString
        MainWindow.TrayIcon.ShowBalloonTip(BalloonsTimeOut)
    End Sub

    Public Sub PlaySound(ByVal SoundFilePath As String)
        If SoundFilePath = "" Then Return
        If Not My.Computer.FileSystem.FileExists(SoundFilePath) Then Exit Sub
        On Error Resume Next
        My.Computer.Audio.Play(SoundFilePath, AudioPlayMode.Background)
    End Sub

    Public Sub ShowWindows()
        MainWindow.Show()
        For Each objApp As CApplication In Apps
            objApp.Show()
        Next
    End Sub

    Public Sub HideWindows()
        MainWindow.Hide()
        For Each objApp As CApplication In Apps
            objApp.Hide()
        Next
    End Sub
End Module
